@@ -15,7 +15,7 @@ except:
 # In[3]:
 
 
-version = '0.60'
+version = '0.61'
 
 import re
 import tempfile
@@ -90,6 +90,9 @@ from urllib.request import urlopen
 requirements.append('ipython')
 from IPython.core.display import display, HTML
 
+requirements.append('pip install PyYAML')
+import yaml
+
 
 # # Strings
 
@@ -333,11 +336,38 @@ if is_testing():
     print(txtread('https://arxiv.org'))
 
 
-# # List of Strings
+# # YAML files
 
 # In[29]:
 
 
+def yamlwrite(path, obj):
+    txtwrite(path, yaml.dump(obj))
+
+
+# In[30]:
+
+
+def yamlread(path):
+    return yaml.safe_load(txtread(path=path))
+
+
+# In[31]:
+
+
+if is_testing():
+    with tempfile.TemporaryDirectory() as dir_path:
+        out_path = dir_path + '/test.yml'
+        
+        yamlwrite(out_path, {'a':['b', 'c']})
+        print(yamlread(out_path))
+
+
+# # List of Strings
+
+# In[32]:
+
+
 regex_cache = {}
 def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
     regex_cache[re_include] = regex_cache.get(re_include, re.compile(re_include))
@@ -347,35 +377,35 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[30]:
+# In[33]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[31]:
+# In[34]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[32]:
+# In[35]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[33]:
+# In[36]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[34]:
+# In[37]:
 
 
 if is_testing():
@@ -384,7 +414,7 @@ if is_testing():
 
 # # GCP Google Cloud Platform
 
-# In[35]:
+# In[38]:
 
 
 import warnings
@@ -408,7 +438,7 @@ def _gsBlob(path):
     return blob
 
 
-# In[36]:
+# In[39]:
 
 
 def gsGsToHttp(path):
@@ -418,7 +448,7 @@ def gsHttpToGs(path):
     return path.replace("https://storage.cloud.google.com/", "gs://").split("?")[0] # "?" is for possible "?authuser=0"
 
 
-# In[37]:
+# In[40]:
 
 
 def gsLs(path):
@@ -428,7 +458,7 @@ def gsLs(path):
     return ['gs://' + bucket + '/' + blob.name for blob in client.list_blobs(prefix=prefix)]
 
 
-# In[38]:
+# In[41]:
 
 
 def gsExists(path):
@@ -438,7 +468,7 @@ def gsExists(path):
     return client.blob(prefix).exists()
 
 
-# In[39]:
+# In[42]:
 
 
 def gsUploadFile(gsPath, path):
@@ -446,7 +476,7 @@ def gsUploadFile(gsPath, path):
     blob.upload_from_filename(path)
 
 
-# In[40]:
+# In[43]:
 
 
 def gsTxtread(path):
@@ -458,7 +488,7 @@ def gsTxtread(path):
     return blob.download_as_string().decode()
 
 
-# In[41]:
+# In[44]:
 
 
 def gsTxtwrite(path, s):
@@ -466,7 +496,7 @@ def gsTxtwrite(path, s):
     return blob.upload_from_string(s)
 
 
-# In[42]:
+# In[45]:
 
 
 def gsImwrite(path, img):
@@ -475,7 +505,7 @@ def gsImwrite(path, img):
     blob.upload_from_string(encimg.tobytes())
 
 
-# In[43]:
+# In[46]:
 
 
 def gsImread(path):
@@ -484,7 +514,7 @@ def gsImread(path):
     return cv2.imdecode(raw, 1)[:, :, [2, 1, 0]]
 
 
-# In[44]:
+# In[47]:
 
 
 def gsPklWrite(path, obj):
@@ -492,7 +522,7 @@ def gsPklWrite(path, obj):
     return blob.upload_from_string(pickle.dumps(obj, protocol=4))
 
 
-# In[45]:
+# In[48]:
 
 
 def gsPklRead(path):
@@ -502,7 +532,7 @@ def gsPklRead(path):
 
 # # Pickle
 
-# In[46]:
+# In[49]:
 
 
 def pklwrite(path, obj):
@@ -513,7 +543,7 @@ def pklwrite(path, obj):
             return pickle.dump(obj, f, protocol=4)
 
 
-# In[47]:
+# In[50]:
 
 
 def pklread(path):
@@ -524,7 +554,7 @@ def pklread(path):
             return pickle.load(f)
 
 
-# In[48]:
+# In[51]:
 
 
 if is_testing():
@@ -537,7 +567,7 @@ if is_testing():
 
 # # Images
 
-# In[49]:
+# In[52]:
 
 
 def imread(path):
@@ -571,7 +601,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255):
     return np.pad(img, [(top, bottom), (left, right), (0, 0)], 'constant', constant_values=color)
 
 
-# In[50]:
+# In[53]:
 
 
 def imshow(array, scale=1):
@@ -586,7 +616,7 @@ def imshow(array, scale=1):
         cv2.waitKey()
 
 
-# In[51]:
+# In[ ]:
 
 
 if is_testing():
@@ -594,14 +624,14 @@ if is_testing():
     imshow(img)
 
 
-# In[52]:
+# In[ ]:
 
 
 if is_testing():
     imshow('http://via.placeholder.com/70.png')
 
 
-# In[53]:
+# In[ ]:
 
 
 def imscaleNN(img, s):
@@ -611,7 +641,7 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[54]:
+# In[ ]:
 
 
 if is_testing():
@@ -630,7 +660,7 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[55]:
+# In[ ]:
 
 
 def imRepeat(img, height, width):
@@ -644,7 +674,7 @@ def imRepeat(img, height, width):
     return img
 
 
-# In[56]:
+# In[ ]:
 
 
 if is_testing():
@@ -652,33 +682,33 @@ if is_testing():
     imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
 
 
-# In[57]:
+# In[ ]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[58]:
+# In[ ]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[59]:
+# In[ ]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[60]:
+# In[ ]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[61]:
+# In[ ]:
 
 
 def imAddNoiseGauss(img, std):
@@ -690,7 +720,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
 
 
-# In[62]:
+# In[ ]:
 
 
 def imJpgDegradation(img, quality):
@@ -703,7 +733,7 @@ def imJpgDegradation(img, quality):
 if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[63]:
+# In[ ]:
 
 
 def imCropCenter(img, size):
@@ -718,35 +748,35 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[64]:
+# In[ ]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[65]:
+# In[ ]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[66]:
+# In[ ]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[67]:
+# In[ ]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[68]:
+# In[ ]:
 
 
 def imGallery(imgs, pad=0):
@@ -777,7 +807,7 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[69]:
+# In[ ]:
 
 
 if is_testing():
@@ -785,42 +815,42 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[70]:
+# In[ ]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[71]:
+# In[ ]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[72]:
+# In[ ]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[73]:
+# In[ ]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[74]:
+# In[ ]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[75]:
+# In[ ]:
 
 
 if is_testing():
